/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "utils.h"

#include <QDir>
#include <QJsonDocument>
#include <QJsonObject>
#include <QStandardPaths>

Utils::Utils(QObject *parent)
    : QObject(parent)
{
}

Utils::~Utils()
{

}

Utils *Utils::getInstance()
{
    static Utils instance;
    return &instance;
}

QString Utils::mkMutiDir(const QString &path)
{
    QDir dir(path);
    if (path.isEmpty() || dir.exists()) {
        return path;
    }
    QString parentDir = mkMutiDir(path.mid(0, path.lastIndexOf('/')));
    QString dirname = path.mid(path.lastIndexOf('/') + 1);
    QDir parentPath(parentDir);
    if (!dirname.isEmpty())
        parentPath.mkpath(dirname);
    return parentDir + "/" + dirname;
}

bool Utils::parseVoiceInfo(const QString &jsonData, VoiceInfo &info)
{
    QJsonDocument noteDoc = QJsonDocument::fromJson(jsonData.toUtf8());
    if (noteDoc.toJson().isEmpty()) {
        return false;
    }
    QJsonObject note = noteDoc.object();
    info.title = note.value("title").toString();
    info.voicePath = getAudioSavePath() + note.value("voicePath").toString();
    info.voiceSize = note.value("voiceSize").toInt();
    return true;
}

QString Utils::makeVoiceInfo(const VoiceInfo &info)
{
    QJsonDocument noteDoc;
    QJsonObject note;
    note.insert("title", info.title);
    note.insert("voicePath", info.voicePath);
    note.insert("voiceSize", info.voiceSize);
    note.insert("createTime", info.createTime.toString("yyyy/MM/dd hh:mm"));
    noteDoc.setObject(note);
    return noteDoc.toJson(QJsonDocument::Compact);
}

qreal Utils::getAudioWaveform(const QAudioBuffer &data, qint64 offset)
{
    qreal ret(0);
    QAudioFormat format = data.format();
    int channelData = data.sampleCount() / format.channelCount();
    if (offset > channelData) {
        return  ret;
    }
    if (format.isValid()) {
        switch (format.sampleType()) {
        case QAudioFormat::Unknown:
            break;
        case QAudioFormat::Float:
            if (format.sampleSize() != 32) // other sample formats are not supported
                return  ret;
            else
                ret = 1.00003;
            break;
        case QAudioFormat::SignedInt:
            if (format.sampleSize() == 32)
#ifdef Q_OS_WIN
                ret = INT_MAX;
#endif
#ifdef Q_OS_UNIX
            ret = SHRT_MAX;
#endif
            else if (format.sampleSize() == 16)
                ret = SHRT_MAX;
            else if (format.sampleSize() == 8)
                ret = CHAR_MAX;
            break;
        case QAudioFormat::UnSignedInt:
            if (format.sampleSize() == 32)
                ret = UINT_MAX;
            else if (format.sampleSize() == 16)
                ret = USHRT_MAX;
            else if (format.sampleSize() == 8)
                ret = UCHAR_MAX;
            break;
        }
    }
    const qint16 *tmpData = data.constData<qint16>();
    return  tmpData[channelData + offset] / ret;
}

QString Utils::getAudioSavePath()
{
    QString path = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/voicenote/";
    return  mkMutiDir(path);
}
