/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
* Author:     zhangzhong <zhangzhong@deepin.io>
*
* Maintainer: zhangzhong <zhangzhong@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "datalistmodel.h"
#include <QMetaEnum>
#include <QTextDocument>

DataListModel::DataListModel()
{
    //初始化
    m_srcdata = new QStandardItemModel(this);

    //绑定数据
    this->setSourceModel(m_srcdata);
    this->setDynamicSortFilter(false);

    m_updateTime = new QTimer(this);
    connect(m_updateTime, SIGNAL(timeout()), this, SLOT(handleTimeout()));
    connect(DataManager::getInstance(), &DataManager::sigLoadAllNoteContentFinsh, [&]() {m_noteContentReady = true;});
    m_updateTime->start(1000 * 60);
}

DataListModel::~DataListModel()
{
}

void DataListModel::handleTimeout()
{
    if (this->rowCount() >= 1) {
        const QModelIndex begindata = this->index(0, 0);
        const QModelIndex enddata = this->index(this->rowCount() - 1, 0);
        emit dataChanged(begindata, enddata);
    }
}

QVariant DataListModel::slotGetItem(const int parIndex, ItemRoles tagRoles)
{
    return this->data(this->index(parIndex, 0), tagRoles);
}

QStandardItem *DataListModel::createStandardItem(void *data)
{
    QStandardItem *pItem = new QStandardItem;
    pItem->setData(QVariant::fromValue(data), Qt::UserRole);
    return pItem;
}

NoteContent *DataListModel::getStandardItemData(const QModelIndex &index) const
{
    if (!index.isValid()) {
        return nullptr;
    }
    QStandardItem *item = m_srcdata->itemFromIndex(index);
    if (nullptr == item) {
        return  nullptr;
    }
    QVariant var = item->data(Qt::UserRole);
    if (!var.isValid()) {
        return  nullptr;
    }
    return  reinterpret_cast<NoteContent *>(var.value<void *>());
}

bool DataListModel::lessThan(
    const QModelIndex &source_left,
    const QModelIndex &source_right) const
{
    NoteContent *leftNote = getStandardItemData(source_left);
    NoteContent *rightNote = getStandardItemData(source_right);

    if (nullptr != leftNote && nullptr != rightNote) {
        if (leftNote->top != rightNote->top) {
            return leftNote->top ? false : true;
        }

        switch (m_sortFeild) {
        case NoteListViewSortModel::ModifyTime:
            return (leftNote->modifyTime < rightNote->modifyTime);
        case NoteListViewSortModel::CreationTime:
            return (leftNote->createTime < rightNote->createTime);
        case NoteListViewSortModel::NoteName:
            return (leftNote->name > rightNote->name);
        case NoteListViewSortModel::DeteleTime:
            return (leftNote->deleteTime < rightNote->deleteTime);
        case NoteListViewSortModel::Customize:
            //自定义 暂定
            return leftNote->customSortNum > rightNote->customSortNum;
        }
    }

    return QSortFilterProxyModel::lessThan(source_left, source_right);
}

bool DataListModel::addNote(qint64 folderContentId)
{
    bool bRet = false;
    QString strNoteTitle = QString(tr("Untitled note"));
    QString strNoteContent = "<p style=\"text-align: left;\" data-mce-style=\"text-align: left;\"><br data-mce-bogus=\"1\"></p>";
    QString strSynopsisContent = QString();
    QString strSynopsisIcon = QString();

    NoteContent *noteContent = DataManager::getInstance()->addNote(noteContentType::NOTEHTMLTEXT, strNoteTitle, \
                                                                   strNoteContent, strSynopsisContent, strSynopsisIcon, folderContentId);
    if (noteContent) {
        QStandardItem *item = createStandardItem(noteContent);
        m_srcdata->appendRow(item);
        bRet = true;
    }

    return bRet;
}

int DataListModel::deleteNote(int noteItemIndex, bool isRecycle)
{
    int iRet = DeleteNoteRet::DeleteNote_Faid;

    QModelIndex modelIndex = this->index(noteItemIndex, 0);
    NoteContent *noteContent = getStandardItemData(mapToSource(modelIndex));

    if (noteContent) {
        if (DataManager::getInstance()->deleteNote(noteContent, isRecycle)) {
            this->removeRow(noteItemIndex);
            iRet = DeleteNoteRet::DeleteNote_Succeed;
        }
    }

    if (this->rowCount() == 0) {
        iRet = DeleteNoteRet::DeleteNote_Notebook_Null;
    }

    return iRet;
}

int DataListModel::sortView(int currentIndex)
{
    int iID = -1;
    int iNewCurrent = -1;
    //判断当前的INDEX值
    if (currentIndex >= 0) {
        iID = (slotGetItem(currentIndex, IdRole)).toInt();
    }

    bool isRecycle = false;
    //判断是否为回收站目录
    if (m_folder->type == RECYCLEROOT) {
        isRecycle = true;
    }

    InteractionManager::NoteSortType noteSortType = InteractionManager::instance()->getSortType(isRecycle);
    m_sortFeild = noteSortType.sortModel;
    sort(0, noteSortType.bIsInverted);

    //查找新的INDEX
    if (iID != -1) {
        for (int i = 0; i < this->rowCount(); ++i) {
            if (iID == (slotGetItem(i, IdRole)).toInt()) {
                iNewCurrent = i;
                break;
            }
        }
    }

    return iNewCurrent;
}

bool DataListModel::editItemSource(qint64 folderContentId, int selecIndex)
{
    //删除源数据
    if (m_srcdata != nullptr) {
        m_srcdata->clear();
    }

    bool bRet {true};
    FolderContent *pFolder = DataManager::getInstance()->getFolder(folderContentId);
    if (pFolder) {
        //记录当前显示的目录
        m_folder = pFolder;
        //重置标记
        m_bDoSort = false;
        QList<NoteContent *> dataList = DataManager::getInstance()->getNotes(pFolder);
        if (!dataList.isEmpty()) {
            for (int i = 0; i < dataList.size(); i++) {
                NoteContent *note = dataList[i];
                QStandardItem *item = createStandardItem(note);
                m_srcdata->appendRow(item);
            }
        } else {
            bRet = false;
        }
        //执行排序
        if (bRet) {
            sortView(selecIndex);
        }
    } else {
        bRet = false;
    }

    return bRet;
}

int DataListModel::getFolderContentType(qint64 folderContentId)
{
    int iFolderContentType = 0;
    FolderContent *pFolderContent = DataManager::getInstance()->getFolder(folderContentId);
    if (pFolderContent) {
        iFolderContentType = pFolderContent->type;
    }

    return iFolderContentType;
}

int DataListModel::restoreNote(const QVariantList &indexList)
{
    int bRet = DeleteNoteRet::DeleteNote_Faid;
    QModelIndex modelIndex;
    NoteContent *noteContent = nullptr;

    for (auto index : indexList) {
        int noteItemIndex = index.toInt();
        modelIndex = this->index(noteItemIndex, 0);
        noteContent = getStandardItemData(mapToSource(modelIndex));
        if (noteContent) {
            if (DataManager::getInstance()->updateParent(noteContent, 2)) {
                this->removeRow(noteItemIndex);
                bRet = DeleteNoteRet::DeleteNote_Succeed;
            }

            if (!DataManager::getInstance()->updateDeleteFlag(noteContent, false)) {
                bRet = DeleteNoteRet::DeleteNote_Faid;
            }
        }
    }

    if (bRet == DeleteNoteRet::DeleteNote_Succeed) {
        if (DataManager::getInstance()->getRecycleNotes().isEmpty()) {
            bRet = DeleteNoteRet::DeleteNote_Notebook_Null;
        }
    }

    return bRet;
}

void DataListModel::onDataUpdate(const int index)
{
    const QModelIndex begindata = this->index(index, 0);
    emit dataChanged(begindata, begindata);
}

int DataListModel::checkNeedSort(const int index)
{
    bool isRecycle = false;
    //判断是否为回收站目录
    if (m_folder->type == RECYCLEROOT) {
        isRecycle = true;
    }

    InteractionManager::NoteSortType noteSortType = InteractionManager::instance()->getSortType(isRecycle);
    m_sortFeild = noteSortType.sortModel;
    if ((NoteListViewSortModel::ModifyTime == m_sortFeild && !m_bDoSort) || NoteListViewSortModel::NoteName == m_sortFeild) {
        m_bDoSort = true;
        return sortView(index);
    }

    return -1;
}

void DataListModel::resetDoSort()
{
    m_bDoSort = false;
}

bool DataListModel::clearRecylce()
{
    m_srcdata->clear();
    bool bRet = DataManager::getInstance()->clearRecycle();
    return bRet;
}

QHash<int, QByteArray> DataListModel::roleNames() const
{
    QHash<int, QByteArray> roles = QSortFilterProxyModel::roleNames();
    QMetaEnum m = QMetaEnum::fromType<DataListModel::ItemRoles>();
    for (int i = 0; i < m.keyCount(); ++i) {
        roles[m.value(i)] = m.key(i);
    }
    return roles;
}

QVariant DataListModel::data(const QModelIndex &index, int role) const
{
    if (role < DelFlagRole || role > DisplayTimeRole) {
        return QSortFilterProxyModel::data(index, role);
    }
    NoteContent *note = getStandardItemData(mapToSource(index));
    if (nullptr == note) {
        return QSortFilterProxyModel::data(index, role);
    }

    switch (role) {
    case DelFlagRole:       return note->deleteFlag;
    case IdRole:            return note->id;
    case ParentIdRole:      return note->parentId;
    case NameRole:          return note->name;
    case PasswordRole:      return note->password;
    case CreateTimeRole:    return timeConversion(note->createTime);
    case ModifyTimeRole:    return timeConversion(note->modifyTime);
    case DelTimeRole:       return timeConversion(note->deleteTime);
    case TypeRole:          return note->type;
    case TopRole:           return note->top;
    case FontsizeRole:      return note->fontSize;
    case FontnameRole:      return note->fontName;
    case SynContentRole:    return note->synopsisContent;
    case SynImgRole:        return note->synopsisIcon;
    case ContentRole:       return note->content;
    case TagsRole:          return note->tags.join(",");
    case TopTimeRole:       return note->topTime.toString("yyyy-MM-dd hh:mm:ss");
    case TopLineRole: {
        return showTopLineForIndex(index);
    }
    case DisplayTimeRole: {
        if (InteractionManager::instance()->getNoteSortModel(m_folder->type == RECYCLEROOT) == InteractionManager::ModifyTime) {
            return timeConversion(note->modifyTime);
        } else if (InteractionManager::instance()->getNoteSortModel(m_folder->type == RECYCLEROOT) == InteractionManager::DeteleTime) {
            return timeConversion(note->deleteTime);
        } else {
            return timeConversion(note->createTime);
        }
    }
    }
    return QSortFilterProxyModel::data(index, role);
}

bool DataListModel::filterAcceptsRow(int source_row, const QModelIndex &source_parent) const
{
    QModelIndex index = sourceModel()->index(source_row, 0 , source_parent);
    return searchKey(getStandardItemData(index));
}

QString DataListModel::timeConversion(QDateTime &pTimer) const
{
    QDateTime currTime = QDateTime::currentDateTime();
    qint64 intervalTime = pTimer.secsTo(currTime);

    bool bSameYear = false;
    if (pTimer.date().year() == currTime.date().year()) {
        bSameYear = true;
    }

    QString strTip = "";
    if (intervalTime < 60) {
        strTip = tr("Just now");
    } else if (intervalTime >= 60 && intervalTime < 60 * 60) {
        qint64 minBefore = intervalTime / 60;
        if (minBefore == 1) {
            strTip = tr("1 min ago");
        } else {
            strTip = tr("%1 mins ago").arg(QString::number(minBefore));
        }
    } else if (intervalTime >= 60 * 60 && intervalTime < 60 * 60 * 24) {
        qint64 hourBefore = intervalTime / 3600;
        if (hourBefore == 1) {
            strTip = tr("1 hour ago");
        } else {
            strTip = tr("%1 hours ago").arg(QString::number(hourBefore));
        }
    } else if (intervalTime >= 60 * 60 * 24 && intervalTime < 60 * 60 * 24 * 2) {
        strTip = tr("Yesterday");
    } else if (intervalTime >= 60 * 60 * 24 * 2 && bSameYear) {
        strTip = pTimer.toString("MM-dd");
    } else {
        strTip = pTimer.toString("yyyy-MM-dd");
    }
    return strTip;
}

bool DataListModel::setNoteTop(int noteItemIndex)
{
    QModelIndex modelIndex = this->index(noteItemIndex, 0);
    NoteContent *noteContent = getStandardItemData(mapToSource(modelIndex));

    bool bTopEx = !noteContent->top;
    if (noteContent) {
        if (DataManager::getInstance()->updateTopFlag(noteContent, bTopEx)) {
            //更新置顶数据
            onDataUpdate(noteItemIndex);
            return true;
        }
    }
    return false;
}

bool DataListModel::getNoteListTop()
{
    for (int i = 0; i < this->rowCount(); ++i) {
        QModelIndex modelIndex = this->index(i, 0);
        NoteContent *noteContent = getStandardItemData(mapToSource(modelIndex));
        if (noteContent->top) {
            return true;
        }
    }
    return false;
}

bool DataListModel::loadSearchNotes(const QString &key, const int searchType)
{
    setSearchKey(key);
    if (searchType == InteractionManager::NOTEBOOKROOT) {
        m_srcdata->clear();

        QList<NoteContent *> allNotes = DataManager::getInstance()->getAllNotes();
        for (auto it : allNotes) {
            if (searchKey(it, std::bind(&DataManager::loadAllNoteContent, DataManager::getInstance()))) {
                m_srcdata->appendRow(createStandardItem(it));
            }
        }

        sortView(0);
    } else {
        invalidate();
    }
    return rowCount() == 0 ? false : true;
}

bool DataListModel::setSearchKey(const QString &key)
{
    m_searchKey = key;
}

bool DataListModel::showTopLineForIndex(const QModelIndex &index) const
{
    NoteContent *note = getStandardItemData(mapToSource(index));
    if (note && note->top) {
        int iIndexNext = index.row() + 1;
        if (iIndexNext >= this->rowCount()) {
            return true;
        } else {
            QModelIndex modelNext = this->index(iIndexNext, 0);
            NoteContent *noteNext = getStandardItemData(mapToSource(modelNext));
            if (!noteNext->top) {
                return true;
            }
        }
    }
    return false;
}


bool DataListModel::searchKey(const NoteContent *note, std::function<void ()> callback) const
{
    if (note->name.contains(m_searchKey, Qt::CaseInsensitive)) {
        return true;
    }

    if (!note->synopsisContent.isEmpty()) {
        return note->synopsisContent.contains(m_searchKey, Qt::CaseInsensitive);
    }

    if (!m_noteContentReady && callback != nullptr) {
        callback();
    }

    if (!note->content.isEmpty()) {
        QTextDocument doc;
        doc.setHtml(note->content);
        return doc.toPlainText().contains(m_searchKey, Qt::CaseInsensitive);

    }

    return false;
}


