/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
* Author:     zhangzhong <zhangzhong@deepin.io>
*
* Maintainer: zhangzhong <zhangzhong@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DATALISTMODEL_H
#define DATALISTMODEL_H
#include "datamanager.h"
#include "interactionmanager.h"

#include <QObject>
#include <QDebug>
#include <QAbstractListModel>
#include <QStandardItemModel>
#include <QSortFilterProxyModel>
#include <QTimer>

class DataListModel : public QSortFilterProxyModel
{
    Q_OBJECT
    typedef InteractionManager::NoteListViewSortModel  NoteListViewSortModel;

public:
    enum ItemRoles {
        DelFlagRole = Qt::UserRole + 10,
        IdRole,
        ParentIdRole,
        NameRole,
        PasswordRole,
        CreateTimeRole,
        ModifyTimeRole,
        DelTimeRole,
        TypeRole,
        TopRole,
        FontsizeRole,
        FontnameRole,
        SynContentRole,
        SynImgRole,
        ContentRole,
        TagsRole,
        TopTimeRole,
        TopLineRole,
        DisplayTimeRole
    };
    Q_ENUM(ItemRoles)

    enum DeleteNoteRet {
        DeleteNote_Faid = 0,
        DeleteNote_Succeed,
        DeleteNote_Notebook_Null
    };

    explicit DataListModel();
    ~DataListModel();

    /**
     * @brief createStandardItem
     * @param index 索引
     * @return 数据项类型
     */
    QStandardItem *createStandardItem(void *data);

    /**
     * @brief getStandardItemData
     * @param index
     * @return 数据项绑定的数据
     */
    NoteContent *getStandardItemData(const QModelIndex &index) const;

    /**
     * @brief MiddleViewSortFilter::sortView
     * @param currentIndex 当前的INDEX值
     */
    Q_INVOKABLE int sortView(int currentIndex);
    /**
     * @brief MiddleViewSortFilter::lessThan  处理排序
     * @param source_left
     * @param source_right
     * @return true source_left小于source_right
     */
    virtual bool lessThan(
        const QModelIndex &source_left,
        const QModelIndex &source_right) const override;

    /**
     * @brief slotGetItem 获取item数据
     * @param parIndex 序号 tagRoles 数据类型
     * @return 数据
     */
    Q_INVOKABLE QVariant slotGetItem(const int parIndex, ItemRoles tagRoles);

    /**
     * @brief addNote 新增笔记
     * @param folderContentId 笔记本id
     * @return
     */
    Q_INVOKABLE bool addNote(qint64 folderContentId);

    /**
     * @brief addNote 删除笔记
     * @param noteItemIndex 笔记item index，isRecycel 是否是回收站笔记
     * @return 0:删除失败，1:删除成功，2:笔记本被删空
     */
    Q_INVOKABLE int deleteNote(int noteItemIndex, bool isRecycle);

    /**
     * @brief editItemSource 修改数据源
     * @param folderContentId 笔记本id, selecIndex 当前选择项
     * @return true：有数据源，false：无数据源
     */
    Q_INVOKABLE bool editItemSource(qint64 folderContentId, int selecIndex);

    /**
     * @brief getFolderContentType 获取笔记本类型
     * @param folderContentId 笔记本id
     * @return 笔记本类型
     */
    Q_INVOKABLE int getFolderContentType(qint64 folderContentId);

    /**
     * @brief restoreNote
     * @param indexList 选中的笔记列表
     * @return
     */
    Q_INVOKABLE int restoreNote(const QVariantList &indexList);

    /**
     * @brief deleteRecylce 删除回收站
     * @return
     */
    Q_INVOKABLE bool clearRecylce();

    /**
     * @brief onDataUpdate 数据发生变动
     * @return
     */
    Q_INVOKABLE void onDataUpdate(const int index);

    /**
     * @brief checkNeedSort 检测是否需要排序
     * @return int 返回新的INDEX
     */
    Q_INVOKABLE int checkNeedSort(const int index);

    /**
     * @brief resetDoSort 当前编辑的文档发生变动时，重置标记
     * @return
     */
    Q_INVOKABLE void resetDoSort();

    /**
     * @brief setNoteTop 设置文档置顶
     * @param bTop 是否置顶
     * @return
     */
    Q_INVOKABLE bool setNoteTop(int noteItemIndex);

    /**
     * @brief getNoteListTop 获取列表中是否有置顶数据
     * @param
     * @return 是否有置顶数据
     */
    Q_INVOKABLE bool getNoteListTop();

    /**
     * @brief loadSearchNotes 加载符合条件的搜索笔记
     * @param key 搜索的关键字
     * @param searchType 搜索的笔记类型
     * @param id 笔记id
     * @return false 搜索无结果 true 搜索有结果
     */
    Q_INVOKABLE bool loadSearchNotes(const QString &key, const int searchType);

    /**
     * @brief setSearchKey
     * @param key
     * @return
     */
    Q_INVOKABLE bool setSearchKey(const QString &key);

protected:
    QHash<int, QByteArray> roleNames() const override;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
    bool filterAcceptsRow(int source_row, const QModelIndex &source_parent) const override;

private slots:
    /**
     * @brief handleTimeout 超时处理
     * @return
     */
    void handleTimeout();

private:
    /**
     * @brief timeConversion 时间转换
     * @return 当前显示的时间
     */
    QString timeConversion(QDateTime &pTimer) const;

    /**
     * @brief showTopLineForIndex 是否显示置顶
     * @return 当前显示的时间
     */
    bool showTopLineForIndex(const QModelIndex &index) const;


    /**
     * @brief searchKey 搜索关键字
     * @param key 关键字
     * @param note 笔记
     * @return
     */
    bool searchKey(const NoteContent *note, std::function<void ()> resultCallback = nullptr) const;

    //根目录对象
    FolderContent *m_folder       {nullptr};
    //原始数据
    QStandardItemModel *m_srcdata {nullptr};
    //排序规则
    NoteListViewSortModel m_sortFeild         { NoteListViewSortModel::CreationTime };

    //是否已经执行过排序(作用是当前为修改时间排序时，只需要执行一次排序)
    bool m_bDoSort = false;

    //刷新时间定时器
    QTimer *m_updateTime;
    // 搜索关键字
    QString m_searchKey {""};
    // 笔记内容是否加载完成
    bool m_noteContentReady {false};
};

#endif // DATALISTMODEL_H

