/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "jscontent.h"
#include "notedata.h"
#include "datamanager.h"
#include "utils.h"
#include "dbusconnectinfo.h"

#include <QStandardPaths>
#include <QFileInfo>
#include <QImage>
#include <QClipboard>
#include <QApplication>
#include <QMimeData>
#include <QUrl>
#include <QDesktopServices>
#include <QFileInfo>
#include <QDir>

JSContent::JSContent(QObject *parent) : QObject(parent)
{

}

void JSContent::setNoteId(qint64 id)
{
    if (!m_note || m_note->id != id) {
        m_note = DataManager::getInstance()->getNote(id);
        slotFillWebContent();
        emit sigNoteIdChange(m_note ? id : -1);
    }
}

qint64 JSContent::getNoteId()
{
    if (nullptr != m_note) {
        return m_note->id;
    }
    return -1;
}

bool JSContent::slotUpdateNoteContent(const QString &content, const QString &synopsis_content, const QString &synopsis_icon)
{
    QString synContent = synopsis_content;
    synContent.replace("\n"," ");
    if (DataManager::getInstance()->updateContent(m_note, content, synContent, synopsis_icon)) {
        emit sigNoteContentChange(m_note->id);
        return true;
    }
    return false;
}

bool JSContent::slotUpdateNoteName(const QString &name)
{
    if (DataManager::getInstance()->updateName(m_note, name)) {
        emit sigNoteNameChange(m_note->id);
        return true;
    }
    return false;
}

bool JSContent::slotFillWebContent()
{
    if (nullptr != m_note) {
        emit sigCallJsSetReadonly(m_note->deleteFlag);
        if (m_note->content.isEmpty()) {
            if (!DataManager::getInstance()->fullNoteContent(m_note)) {
                return false;
            }
        }
        switch (m_note->type) {
        case NOTEHTMLTEXT:
            emit sigCallJsSetHtml(m_note->content, m_note->name, m_note->fontName, m_note->fontSize);
            break;
        case NOTEMARKDOWNTEXT:
            emit sigCallJsSetMarkdown(m_note->content, m_note->name, m_note->fontName, m_note->fontSize);
            break;
        case NOTEJSONTEXT:
            emit sigCallJsSetJson(m_note->content, m_note->name, m_note->fontName, m_note->fontSize);
            break;
        }
        return true;
    }
    return false;
}

bool JSContent::slotInsertImages(const QList<QUrl> &files)
{
    int count = 0;
    QStringList paths;
    //获取文件夹路径
    QString dirPath = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/images";
    //创建文件夹
    Utils::getInstance()->mkMutiDir(dirPath);
    //获取时间戳
    QDateTime currentDateTime = QDateTime::currentDateTime();
    QString date = currentDateTime.toString("yyyyMMddhhmmss");

    for (auto file : files) {
        QString path = file.toLocalFile();
        QFileInfo fileInfo(path);
        QString suffix = fileInfo.suffix();
        if (!(suffix == "jpg" || suffix == "png" || suffix == "bmp")) {
            continue;
        }
        //创建文件路径
        QString newPath = QString("%1/%2_%3.%4").arg(dirPath).arg(date).arg(++count).arg(suffix);
        if (QFile::copy(path, newPath)) {
            paths.push_back("file://" + newPath);
        }
    }
    if (paths.size() == 0) {
        return false;
    }
    emit sigCallJsInsertImages(paths);
    return true;
}

bool JSContent::insertImages(const QImage &image)
{
    //获取文件夹路径
    QString dirPath = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/images";
    //创建文件夹
    Utils::getInstance()->mkMutiDir(dirPath);
    //保存文件，文件名为当前年月日时分秒
    QDateTime currentDateTime = QDateTime::currentDateTime();
    QString fileName = currentDateTime.toString("yyyyMMddhhmmss.png");
    QString imgPath = dirPath + "/" + fileName;

    //保存文件
    if (!image.save(imgPath)) {
        return false;
    }
    emit sigCallJsInsertImages(QStringList("file://" + imgPath));
    return true;
}

void JSContent::slotPaste()
{
    QClipboard *clipboard = QApplication::clipboard();
    const QMimeData *mimeData = clipboard->mimeData();
    //存在文件url
    if (mimeData->hasUrls()) {
        slotInsertImages(mimeData->urls());
    } else if (mimeData->hasImage()) {
        insertImages(qvariant_cast<QImage>(mimeData->imageData()));
    } else {
        //无图片文件，直接调用web端的粘贴事件
        emit sigPasteDefault();
    }
}

bool JSContent::slotOpenFile(const QString &file)
{
    return QDesktopServices::openUrl(file);
}

int JSContent::slotCopyFile(const QString &srcFile, const QUrl &dstFile)
{
    QString newPath = dstFile.toLocalFile();
    QString orgPath = QUrl(srcFile).toLocalFile();
    QFileInfo info(newPath);
    if (!QFileInfo(info.dir().path()).isWritable()) {
        return 1;
    }
    if (info.exists() && !QFile::remove(newPath)) {
        return 1;
    }
    if (!QFile::copy(orgPath, newPath)) {
        return 2;
    }
    return 0;
}

void JSContent::slotWriteClipData(const QString &html, QString &text)
{
    QClipboard *clip = QApplication::clipboard();
    if (nullptr != clip) {
        QMimeData *data = new QMimeData;
        data->setData("deepin-note", QByteArray());
        //设置纯文本
        data->setText(text);
        //设置富文本
        data->setHtml(html);
        clip->setMimeData(data);
    }
}

QString JSContent::slotReadClipData(int type)
{
    QClipboard *clip = QApplication::clipboard();
    if (nullptr != clip) {
        const QMimeData *data = clip->mimeData();
        if (nullptr != data) {
            if (0 == type && data->hasHtml()) {
                return data->html();
            } else if (1 == type && data->hasText()) {
                return data->text();
            }
        }
    }
    return QString("");
}

QStringList JSContent::slotGetFontList()
{
    return DbusConnectInfo::instance().GetFontList();
}

QString JSContent::slotGetTranslation()
{
    static QJsonDocument doc;
    if (doc.isEmpty()) {
        QJsonObject object;
        object.insert("Fonts", QApplication::translate("web", "Font"));
        object.insert("Font Sizes", QApplication::translate("web", "Text size"));
        object.insert("Blocks", QApplication::translate("web", "Heading"));
        object.insert("Text color", QApplication::translate("web", "Text color"));
        object.insert("Background color", QApplication::translate("web", "Highlight"));
        object.insert("Bold", QApplication::translate("web", "Bold"));
        object.insert("Italic", QApplication::translate("web", "Italic"));
        object.insert("Underline", QApplication::translate("web", "Underline"));
        object.insert("Strikethrough", QApplication::translate("web", "Strikethrough"));
        object.insert("Align", QApplication::translate("web", "Align"));
        object.insert("Left", QApplication::translate("web", "Align left"));
        object.insert("Center", QApplication::translate("web", "Align center"));
        object.insert("Right", QApplication::translate("web", "Align right"));
        object.insert("More...", "...");
        object.insert("Bullet list", QApplication::translate("web", "Bullets"));
        object.insert("Numbered list", QApplication::translate("web", "Numbering"));
        object.insert("Redo", QApplication::translate("web", "Redo"));
        object.insert("Undo", QApplication::translate("web", "Undo"));
        object.insert("Paragraph", QApplication::translate("web", "Normal"));
        object.insert("Heading 1", QApplication::translate("web", "Heading 1"));
        object.insert("Heading 2", QApplication::translate("web", "Heading 2"));
        object.insert("Heading 3", QApplication::translate("web", "Heading 3"));
        object.insert("Heading 4", QApplication::translate("web", "Heading 4"));
        object.insert("Heading 5", QApplication::translate("web", "Heading 5"));
        object.insert("Heading 6", QApplication::translate("web", "Heading 6"));
        doc.setObject(object);
    }
    return doc.toJson(QJsonDocument::Compact);
}

