/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef JSCONTENT_H
#define JSCONTENT_H

#include <QObject>
#include <QDebug>

struct NoteContent;

class JSContent : public QObject
{
    Q_OBJECT
    Q_PROPERTY(qint64 noteId READ getNoteId WRITE setNoteId NOTIFY sigNoteIdChange)
public:
    explicit JSContent(QObject *parent = nullptr);
    /**
     * @brief setNoteId 设置笔记id
     * @param id 笔记id
     */
    void setNoteId(qint64 id);
    /**
     * @brief getNoteId 获取笔记id
     * @return 笔记id
     */
    qint64 getNoteId();
    /**
     * @brief 插入图片
     */
    bool insertImages(const QImage &image);
signals:
    void sigNoteIdChange(qint64 id);
    /**
     * @brief sigNoteContentChange 笔记内容变化
     * @param id 笔记id
     */
    void sigNoteContentChange(qint64 id);
    /**
     * @brief sigNoteNameChange 笔记标题变化
     * @param id 笔记id
     */
    void sigNoteNameChange(qint64 id);
    /**
     * @brief sigPasteDefault 系统默认粘贴
     */
    void sigPasteDefault();
    /**
     * @brief sigCallJsSetHtml 设置html笔记内容
     * @param text 笔记内容
     * @param name 笔记标题
     * @param fontName 字体名称
     * @param fontName 字号
     */
    void sigCallJsSetHtml(const QString &text, const QString &name, const QString &fontName, qint64 fontSize);
    /**
     * @brief sigCallJsSetMarkdown 设置markdown笔记内容
     * @param text 笔记内容
     * @param name 笔记标题
     * @param fontName 字体名称
     * @param fontName 字号
     */
    void sigCallJsSetMarkdown(const QString &text, const QString &name, const QString &fontName, qint64 fontSize);
    /**
     * @brief sigCallJsSetJson 设置json格式笔记内容
     * @param name 笔记标题
     * @param text 笔记内容
     * @param fontName 字体名称
     * @param fontName 字号
     */
    void sigCallJsSetJson(const QString &text, const QString &name, const QString &fontName, qint64 fontSize);
    /**
     * @brief sigCallJsInsertImages 插入图片
     * @param images 图片路径
     */
    void sigCallJsInsertImages(const QStringList &images);
    /**
     * @brief sigCallJsSetReadonly 设置编辑区只读属性
     * @param flag 标志，true代表只读
     */
    void sigCallJsSetReadonly(bool flag);
public slots:
    /**
     * @brief slotUpdateNoteContent 更新笔记内容
     * @param content 笔记内容
     * @param synopsis_content 摘要内容
     * @param synopsis_icon 摘要图标
     * @return 结果，成功true,失败false
     */
    bool slotUpdateNoteContent(const QString &content, const QString &synopsis_content, const QString &synopsis_icon);
    /**
     * @brief slotUpdateNoteName 更新笔记标题
     * @param name 笔记标题
     * @return 结果，成功true,失败false
     */
    bool slotUpdateNoteName(const QString &name);
    /**
     * @brief fillWebContent 填充web端笔记内容
     * @return 结果，成功true,失败false
     */
    bool slotFillWebContent();
    /**
     * @brief soltPaste 处理粘贴
     */
    void slotPaste();
    /**
       * @brief 插入图片
       * @param filePaths 图片路径
       * @return
       */
    bool slotInsertImages(const QList<QUrl> &filePaths);
    /**
     * @brief slotOpenFile 打开文件
     * @param file 文件路径
     * @return
     */
    bool slotOpenFile(const QString &file);
    /**
     * @brief slotCopyFile 复制文件
     * @param srcFile 原文件
     * @param dstFile 目标文件
     * @return
     */
    int slotCopyFile(const QString &srcFile, const QUrl &dstFile);
    /**
     * @brief slotWriteClip 剪切板写内容
     * @param html 富文本数据
     * @param text 纯文本格式
     */
    void slotWriteClipData(const QString &html, QString &text);
    /**
     * @brief slotReadClipData 获取剪切板内容
     * @param type 0:富文本,1:纯文件
     * @return 内容
     */
    QString slotReadClipData(int type);

    /**
     * @brief slotGetFontList 获取系统字体列表
     * @return 字体信息
     */
    QStringList slotGetFontList();

    /**
     * @brief slotGetTranslation 获取翻译
     * @return 翻译内容
     */
    QString slotGetTranslation();
private:
    NoteContent *m_note {nullptr};
};

#endif // JSCONTENT_H
