/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuxinping <liuxinping@uniontech.com>
*
* Maintainer: liuxinping <liuxinping@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "lefttreemode.h"
#include "datamanager.h"

#include <QtDebug>

LeftTreeMode::LeftTreeMode(QObject *parent)
    : QSortFilterProxyModel(parent)
{
    m_pModel = new QStandardItemModel(this);
    connect(DataManager::getInstance(), &DataManager::sigLoadDataFinsh, this, &LeftTreeMode::slotLoadDataFinsh);
    loadData();
}

LeftTreeMode::~LeftTreeMode()
{
}

void LeftTreeMode::loadData()
{
    //数据加载线程开启
    DataManager::getInstance()->loadNoteByThread();
    DataManager::getInstance()->loadFolderByThread();
}

void LeftTreeMode::initUiData()
{
    //默认一直的有的三项
    FolderContent *folder = DataManager::getInstance()->getQuickNoteRoot();
    folder->name = tr("Quick Notes");
    folder->iconUrl = "quicknotes";
    m_pQuickNoteItem = createStandardItem(folder);

    FolderContent *folderRcy = DataManager::getInstance()->getRecycleNotesRoot();
    folderRcy->name = tr("Trash");
    folderRcy->iconUrl = "trash";
    QStandardItem *item1 = createStandardItem(folderRcy);

    FolderContent *parent = DataManager::getInstance()->getNotebookRoot();
    parent->name = tr("Notebooks");
    parent->iconUrl = "";
    m_pNoteBooksItem = createStandardItem(parent);
    initDbData(m_pNoteBooksItem, DataManager::getInstance()->getNotebookRoot());
    m_pModel->appendRow(m_pQuickNoteItem);
    m_pModel->appendRow(item1);
    m_pModel->appendRow(m_pNoteBooksItem);
    this->setSourceModel(m_pModel);
    this->setDynamicSortFilter(false);
    this->sort(0, Qt::DescendingOrder);
}

void LeftTreeMode::initDbData(QStandardItem *data, FolderContent *folderP)
{
    if (folderP) {
        auto list = DataManager::getInstance()->getChildren(folderP);
        for (int i = 0; i < list.size(); i++) {
            QStandardItem *item1 = createStandardItem(list[i]);
            data->appendRow(item1);
            //递归
            initDbData(item1, list[i]);
        }
    }
}

void LeftTreeMode::deleteData(QStandardItem *data, int row, FolderContent *folderP, QStringList &iter)
{
    if (data == nullptr || folderP == nullptr) return;
    if (folderP) {
        auto list = DataManager::getInstance()->getChildren(folderP);
        for (int i = list.size() - 1; i >= 0; --i) {
            //递归
            deleteData(data->child(i), i, list[i], iter);
        }
        data->parent()->removeRow(row);
        iter.push_back(QString("%1").arg(folderP->id));
    }
}

void LeftTreeMode::slotLoadDataFinsh()
{
    //等待数据加载完成，再初始化treemodel 数据
    initUiData();
}

QStandardItem *LeftTreeMode::createStandardItem(void *data)
{
    QStandardItem *pItem = new QStandardItem;
    pItem->setData(QVariant::fromValue(data), Qt::UserRole + 1);
    return pItem;
}

FolderContent *LeftTreeMode::getStandardItemData(const QModelIndex &index) const
{
    FolderContent *pFolderContent = nullptr;
    QModelIndex ItemIndex = mapToSource(index);
    if (ItemIndex.isValid()) {
        QVariant data = ItemIndex.data(Qt::UserRole + 1);
        if (data.isValid()) {
            pFolderContent = reinterpret_cast<FolderContent *>(data.value<void *>());
        }
    }
    return pFolderContent;
}

bool LeftTreeMode::lessThan(const QModelIndex &source_left, const QModelIndex &source_right) const
{
    //默认排序修改
    FolderContent *folderLeft = reinterpret_cast<FolderContent *>((source_left.data(Qt::UserRole + 1)).value<void *>());
    FolderContent *folderRight = reinterpret_cast<FolderContent *>((source_right.data(Qt::UserRole + 1)).value<void *>());

    if (folderLeft && folderRight) {
        if (folderLeft->type == NOTEBOOKNORMAL)
            return folderLeft->createTime < folderRight->createTime;
        else {
            return folderLeft->customSortNum > folderRight->customSortNum;
        }
    }
    return QSortFilterProxyModel::lessThan(source_left, source_right);
}

QString LeftTreeMode::slotGetItemName(const QModelIndex &index)
{
    FolderContent *folder = getStandardItemData(index);
    if (folder)
        return folder->name;
    return "";
}

QString LeftTreeMode::slotGetItemIcon(const QModelIndex &index)
{
    FolderContent *folder = getStandardItemData(index);
    if (folder)
        return folder->iconUrl;
    return "";
}

QString LeftTreeMode::slotGetItemCount(const QModelIndex &index)
{
    QString  countText = "";
    FolderContent *folder = getStandardItemData(index);
    if (folder) {
        int count = DataManager::getInstance()->getNotes(folder).size();
        if (count)
            countText = QString("%1").arg(count);
    }
    return countText;
}


qint64 LeftTreeMode::slotGetItemFolderId(const QModelIndex &index)
{
    FolderContent *folder = getStandardItemData(index);
    if (folder)
        return folder->id;
    return 0;
}

void LeftTreeMode::slotaddNewNoteBook(const QString &text)
{
    FolderContent *parent = DataManager::getInstance()->getNotebookRoot();
    if (parent == nullptr) return;
    FolderContent *folder = DataManager::getInstance()->addFolder(FolderContentType::NOTEBOOKNORMAL, text, "", parent->id);
    if (folder == nullptr) return;
    QStandardItem *item = createStandardItem(folder);
    m_pNoteBooksItem->appendRow(item);
    this->sort(0, Qt::DescendingOrder);
    QModelIndex indexs = mapFromSource(m_pNoteBooksItem->child(0, 0)->index());
    emit signalCreateNoticeToQml(indexs.sibling(0, 0));
}

void LeftTreeMode::slotAddChilderNode(const QModelIndex &index, const QString &text)
{
    if (m_pModel) {
        QStandardItem *itemParent = m_pModel->itemFromIndex(mapToSource(index));
        FolderContent *folderParent = getStandardItemData(index);
        if (folderParent == nullptr) return;
        FolderContent *folder = DataManager::getInstance()->addFolder(FolderContentType::NOTEBOOKNORMAL, text, "", folderParent->id);
        if (folder == nullptr) return;
        QStandardItem *item = createStandardItem(folder);
        if (itemParent)
            itemParent->appendRow(item);
        this->sort(0, Qt::DescendingOrder);
        QModelIndex indexs = mapFromSource(m_pModel->index(0, 0, mapToSource(index)));
        emit signalCreateNoticeToQml(indexs.sibling(0, 0));
    }
}

void LeftTreeMode::slotRenameNoteBooks(const QModelIndex &index, const QString &text)
{
    if (m_pModel) {
        QStandardItem *itemParent = m_pModel->itemFromIndex(mapToSource(index));
        FolderContent *folderParent = getStandardItemData(index);
        if (folderParent == nullptr) return;
        bool bRet = DataManager::getInstance()->updateName(folderParent, text);
        if (!bRet) {
            qInfo() << "folder rename id failed---------------";
        }
        folderParent->name = text;
        void *data = folderParent;
        itemParent->setData(QVariant::fromValue(data), Qt::UserRole + 1);
        this->sort(0, Qt::DescendingOrder);
    }
}

void LeftTreeMode::slotDeleteNoteBookFolder(const QModelIndex &index)
{
    if (m_pModel) {
        //获取当前选项的同级项个数
        int row = m_pModel->itemFromIndex(mapToSource(index).parent())->rowCount();
        FolderContent *folderParent = getStandardItemData(index);
        if (folderParent == nullptr) return;
        QStandardItem *itemParent = m_pModel->itemFromIndex(mapToSource(index));
        QStringList iter;
        if (itemParent) {
            //递归删除
            deleteData(itemParent, mapToSource(index).row(), folderParent, iter);
            //删除笔记本下所有笔记数据
            bool bRetNote = DataManager::getInstance()->deleteNoteList(iter);
            if (!bRetNote) {
                qInfo() << " delete note data  failed---------------";
            }
            //删除笔记本下所有笔记本数据
            bool bRet = DataManager::getInstance()->deleteFolder(iter);
            if (!bRet) {
                qInfo() << " delete folder data  failed---------------";
            }
        }

        this->sort(0, Qt::DescendingOrder);

        if (row == 1) {
            if (index.parent() != mapFromSource(m_pNoteBooksItem->index()))
                emit signalDeleteNoticeToQml(index.parent());
            else
                emit signalDeleteNoticeToQml(mapFromSource(m_pQuickNoteItem->index()));
        }
    }
}

void LeftTreeMode::slotupdateData()
{
    this->sort(0, Qt::DescendingOrder);
}

