/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "datamanager.h"
#include "notedata.h"
#include "loaddatathread.h"
#include "db/folderdboper.h"
#include "db/notedboper.h"

#include <QThreadPool>
#include <QDebug>

static NoteDbOper noteDbOper;
static FolderDbOper folderOper;

DataManager::DataManager(QObject *parent)
    : QObject(parent)
{
}

DataManager::~DataManager()
{
    for (auto it : m_folders) {
        delete it;
        it = nullptr;
    }
    m_folders.clear();
    for (auto it : m_notes) {
        delete it;
        it = nullptr;
    }
    m_notes.clear();

    if (nullptr != m_recycleFolder) {
        delete  m_recycleFolder;
        m_recycleFolder = nullptr;
    }
}

DataManager *DataManager::getInstance()
{
    static DataManager instance;
    return  &instance;
}

DataManager::LoadStates DataManager::getLoadStatus()
{
    return m_loadstate;
}

NoteContent *DataManager::getNote(qint64 id)
{
    auto it = m_notes.find(id);
    if (it == m_notes.end()) {
        return nullptr;
    }
    return it.value();
}

NoteContent *DataManager::addNote(noteContentType type, const QString &name, const QString &content, const QString &synopsis_content, const QString &synopsis_icon, qint64 parentId)
{
    NoteContent *note = new NoteContent;
    note->name = name;
    note->type = type;
    note->content = content;
    note->parentId = parentId;
    note->createTime = note->modifyTime = QDateTime::currentDateTime();
    note->synopsisContent = synopsis_content;
    note->synopsisIcon = synopsis_icon;
    if (noteDbOper.addNote(note)) {
        m_notes.insert(note->id, note);
        return note;
    }
    delete note;
    note = nullptr;
    return nullptr;
}

bool DataManager::deleteNote(NoteContent *data, bool isRecycle)
{
    if (noteDbOper.deleteNote(data, isRecycle)) {
        if (false == isRecycle) {
            auto it = m_notes.take(data->id);
            delete it;
            it = nullptr;
        }
        return true;
    }
    return false;
}

bool DataManager::clearRecycle()
{
    if (noteDbOper.clearRecycle()) {
        auto it = m_notes.begin();
        while (it != m_notes.end()) {
            if (it.value()->deleteFlag) {
                delete it.value();
                it.value() = nullptr;
                it = m_notes.erase(it);
            } else {
                ++it;
            }
        }
        return true;
    }
    return false;
}

bool DataManager::deleteNoteList(const QStringList &parentId)
{
    QDateTime time = QDateTime::currentDateTime();
    if (noteDbOper.deleteNoteList(parentId, time)) {
        for (int j = 0; j < parentId.size(); j++) {
            auto iter = getNotes(getFolder(parentId[j].toInt()));
            for (int i = 0; i < iter.size(); i++) {
                iter[i]->deleteFlag = true;
                iter[i]->deleteTime = time;
            }
        }
        return true;
    }
    return false;
}


bool DataManager::updateName(NoteContent *data, const QString &newName)
{
    return noteDbOper.updateName(data, newName);
}

bool DataManager::updateParent(NoteContent *data, qint64 parentid)
{
    return noteDbOper.updateParent(data, parentid);
}

bool DataManager::updateContent(NoteContent *data, const QString &content, const QString &synopsis_content, const QString &synopsis_icon)
{
    return  noteDbOper.updateContent(data, content, synopsis_content, synopsis_icon);
}

bool DataManager::updateDeleteFlag(NoteContent *data, bool flag)
{
    return noteDbOper.updateDeleteFlag(data, flag);
}

bool DataManager::updateTopFlag(NoteContent *data, bool flag)
{
    return noteDbOper.updateTopFlag(data, flag);
}

bool DataManager::fullNoteContent(NoteContent *data)
{
    return  noteDbOper.fullNoteContent(data);
}

void DataManager::loadAllNoteContent()
{
    if (!m_loadstate.testFlag(NoteContentReady)) {
        for (auto it : m_notes) {
            fullNoteContent(it);
        }
        m_loadstate.setFlag(NoteContentReady, true);
    }
    emit sigLoadAllNoteContentFinsh();
}

void DataManager::loadAllNoteContentByThread()
{
    LoadDataThread *worker = new LoadDataThread(this, LoadDataThread::LOADNOTEALLCONTENT);
    worker->setAutoDelete(true);
    QThreadPool::globalInstance()->start(worker, QThread::TimeCriticalPriority);
}

const QList<NoteContent *> DataManager::getAllNotes()
{
    return m_notes.values();
}


FolderContent *DataManager::getFolder(qint64 id)
{
    if (-1 == id) {
        return m_recycleFolder;
    }
    auto it = m_folders.find(id);
    if (it == m_folders.end()) {
        return nullptr;
    }
    return it.value();
}

FolderContent *DataManager::getParent(const ContentBase *node)
{
    return getFolder(node->parentId);
}

FolderContent *DataManager::getNotebookRoot()
{
    FolderContent *folder = nullptr;
    for (auto it : m_folders) {
        if (it->type == NOTEBOOKROOT) {
            folder = it;
            folder->customSortNum = 3;
            break;
        }
    }
    return folder;
}

FolderContent *DataManager::getQuickNoteRoot()
{
    FolderContent *folder = nullptr;
    for (auto it : m_folders) {
        if (it->type == QUICKNOTEROOT) {
            folder = it;
            folder->customSortNum = 1;
            break;
        }
    }
    return folder;
}

FolderContent *DataManager::getRecycleNotesRoot()
{
    if (nullptr == m_recycleFolder) {
        m_recycleFolder = new FolderContent;
        m_recycleFolder->type = RECYCLEROOT;
        m_recycleFolder->customSortNum = 2;
    }
    return m_recycleFolder;
}

QList<FolderContent *>  DataManager::getChildren(FolderContent *folder, bool containsDeleted)
{
    QList<FolderContent *> folders;
    if (!folder->children.isEmpty()) {
        QStringList ids = folder->children.split(",");
        int customSortNum = 0;
        for (auto id : ids) {
            auto it = m_folders.find(id.toLongLong());
            if (it != m_folders.end()) {
                if (!containsDeleted && it.value()->deleteFlag) {
                    continue;
                }
                it.value()->customSortNum = customSortNum++;
                folders.push_back(it.value());
            }
        }
        return folders;
    }

    for (auto it : m_folders) {
        if (it->parentId == folder->id) {
            if (!containsDeleted && it->deleteFlag) {
                continue;
            }
            folders.push_back(it);
        }
    }

    return folders;
}

QList<NoteContent *> DataManager::getNotes(FolderContent *folder, bool containsDeleted)
{
    if (folder->type == RECYCLEROOT) {
        return getRecycleNotes();
    }

    QList<NoteContent *> notes;
    if (!folder->children.isEmpty()) {
        QStringList ids = folder->children.split(",");
        int customSortNum = 0;
        for (auto id : ids) {
            auto it = m_notes.find(id.toLongLong());
            if (it != m_notes.end()) {
                if (!containsDeleted && it.value()->deleteFlag) {
                    continue;
                }
                it.value()->customSortNum = customSortNum++;
                notes.push_back(it.value());
            }
        }
        return notes;
    }
    for (auto it : m_notes) {
        if (it->parentId == folder->id) {
            if (!containsDeleted && it->deleteFlag) {
                continue;
            }
            notes.push_back(it);
        }
    }

    return notes;
}

QList<NoteContent *> DataManager::getRecycleNotes()
{
    QList<NoteContent *> notes;
    for (auto it : m_notes) {
        if (it->deleteFlag) {
            notes.push_back(it);
        }
    }
    return notes;
}

void DataManager::loadFolder()
{
    if (!m_loadstate.testFlag(FolderReady)) {
        m_folders = folderOper.loadFolders();
        if (m_folders.size() == 0) {
            for (int i = NOTEBOOKROOT; i <= QUICKNOTEROOT; i++) {
                addFolder(static_cast<FolderContentType>(i), "", "");
            }
        }
        m_loadstate.setFlag(FolderReady, true);
    }
    if (m_loadstate.testFlag(NoteReady)) {
        emit sigLoadDataFinsh();
    }
}

void DataManager::loadFolderByThread()
{
    LoadDataThread *worker = new LoadDataThread(this, LoadDataThread::LOADFOLEDR);
    worker->setAutoDelete(true);
    QThreadPool::globalInstance()->start(worker, QThread::TimeCriticalPriority);
}

void DataManager::loadNote()
{
    if (!m_loadstate.testFlag(NoteReady)) {
        m_notes = noteDbOper.loadNotes();
        m_loadstate.setFlag(NoteReady, true);
    }
    if (m_loadstate.testFlag(FolderReady)) {
        emit sigLoadDataFinsh();
    }
}

void DataManager::loadNoteByThread()
{
    LoadDataThread *worker = new LoadDataThread(this, LoadDataThread::LOADNOTE);
    worker->setAutoDelete(true);
    QThreadPool::globalInstance()->start(worker, QThread::TimeCriticalPriority);
}

FolderContent *DataManager::addFolder(FolderContentType type, const QString &name, const QString &iconUrl, qint64 parentId)
{
    FolderContent *folder = new FolderContent;
    folder->type = type;
    folder->name = name;
    folder->iconUrl = iconUrl;
    folder->parentId = parentId;
    folder->createTime = folder->modifyTime = QDateTime::currentDateTime();
    if (folderOper.addFolder(folder)) {
        m_folders.insert(folder->id, folder);
        return folder;
    }
    delete  folder;
    folder = nullptr;
    return nullptr;
}

bool DataManager::deleteFolder(const QStringList &data)
{
    if (folderOper.deleteFolder(data)) {
        for (auto id : data) {
            auto it = m_folders.find(id.toInt());
            if (it != m_folders.end()) {
                delete it.value();
                it.value() = nullptr;
                it = m_folders.erase(it);
            }
        }
        return true;
    }
    return false;
}

bool DataManager::updateName(FolderContent *data, const QString &newName)
{
    return folderOper.updateName(data, newName);
}

bool DataManager::updateChildren(FolderContent *data, const QString &newChildren)
{
    return  folderOper.updateChildren(data, newChildren);
}

bool DataManager::updateParent(FolderContent *data, qint64 parentid)
{
    return  folderOper.updateParent(data, parentid);
}



