/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DATAMANAGER_H
#define DATAMANAGER_H
#include "notedata.h"

#include <QObject>
#include <QHash>
#include <QList>

struct ContentBase;
struct NoteContent;
struct FolderContent;

class DataManager : public QObject
{
    Q_OBJECT
public:
    enum LoadState {
        NoteReady = 0x01,
        NoteContentReady = 0x02,
        FolderReady = 0x04,
    };
    Q_DECLARE_FLAGS(LoadStates, LoadState)
    ~DataManager();
    /**
     * @brief getInstance 单例
     * @return 单例对象
     */
    static DataManager *getInstance();
public slots:
    /**
     * @brief getLoadStatus 获取数据加载状态
     * @return 状态
     */
    LoadStates getLoadStatus();
    /**
     * @brief getNote 获取笔记数据
     * @param id 笔记唯一标识
     * @return 笔记数据
     */
    NoteContent *getNote(qint64 id);
    /**
     * @brief addNote 添加笔记
     * @param name 笔记名称
     * @param content 笔记内容
     * @param synopsis_content 内容摘要
     * @param synopsis_icon 内容摘要图标
     * @param parentId 笔记父id
     * @return 笔记数据
     */
    NoteContent *addNote(noteContentType type, const QString &name, const QString &content, const QString &synopsis_content, const QString &synopsis_icon, qint64 parentId);
    /**
     * @brief deleteNote 删除笔记
     * @param data 数据
     * @param isRecycle
     * @return 结果，成功true,失败false
     */
    bool deleteNote(NoteContent *data, bool isRecycle = true);
    /**
     * @brief updateName 更新名称
     * @param data 数据
     * @param newName 新名称
     * @return 结果，成功true,失败false
     */
    bool updateName(NoteContent *data, const QString &newName);
    /**
     * @brief updateParent 更新父节点
     * @param data 数据
     * @param parentid 父节点id
     * @return 结果，成功true,失败false
     */
    bool updateParent(NoteContent *data, qint64 parentid);
    /**
     * @brief updateContent 更新笔记内容
     * @param data 数据
     * @param content 笔记内容
     * @return 结果，成功true,失败false
     */
    bool updateContent(NoteContent *data, const QString &content, const QString &synopsis_content, const QString &synopsis_icon);
    /**
     * @brief updateDeleteFlag 更新删除标志
     * @param data 数据
     * @param flag 标志， true代表删除
     * @return 结果，成功true,失败false
     */
    bool updateDeleteFlag(NoteContent *data, bool flag);

    /**
     * @brief updateTopFlag 更新置顶标志
     * @param data 数据
     * @param flag 标志， true代表置顶
     * @return 结果，成功true,失败false
     */
    bool updateTopFlag(NoteContent *data, bool flag);

    /**
     * @brief clearRecycle清空回收站
     * @return
     */
    bool clearRecycle();

    /**
     * @brief deleteNoteList 根据父id 删除下面的全部笔记
     * @param parentId
     * @return
     */
    bool deleteNoteList(const QStringList &parentId);

    /**
     * @brief fullNoteContent 加载笔记内容
     * @param data 数据
     * @return 结果，成功true,失败false
     */
    bool fullNoteContent(NoteContent *data);
    /**
     * @brief getFolder 获取文件夹数据
     * @param id 唯一标识
     * @return 结果
     */
    FolderContent *getFolder(qint64 id);
    /**
     * @brief getParent 获取父节点
     * @param node 当前节点
     * @return  父节点
     */
    FolderContent *getParent(const ContentBase *node);

    /**
     * @brief getNotebookRoot 获取管理笔记本根节点
     * @return 根节点数据
     */
    FolderContent *getNotebookRoot();
    /**
     * @brief getQuickNoteRoot 获取快速笔记根节点
     * @return
     */
    FolderContent *getQuickNoteRoot();
    /**
     * @brief getRecycleNotesRoot 获取回收站根节点
     * @return
     */
    FolderContent *getRecycleNotesRoot();
    /**
     * @brief addFolder 添加文件夹
     * @param type 类型
     * @param name 名称
     * @param iconUrl 头像路径
     * @param parentId 父id
     * @return 文件夹数据
     */
    FolderContent *addFolder(FolderContentType type, const QString &name, const QString &iconUrl, qint64 parentId = -1);
    /**
     * @brief deleteFolder 删除文件夹
     * @param data 数据
     * @return 结果，成功true,失败false
     */
    bool deleteFolder(const QStringList &data);

    /**
     * @brief updateName 更新名称
     * @param data 数据
     * @param newName 新名称
     * @return 结果，成功true,失败false
     */
    bool updateName(FolderContent *data, const QString &newName);
    /**
     * @brief updateChildren 更新子节点
     * @param data 数据
     * @param newChildren 子节点
     * @return 结果，成功true,失败false
     */
    bool updateChildren(FolderContent *data, const QString &newChildren);
    /**
     * @brief updateParent 更新父节点
     * @param data 数据
     * @param parentid 父节点
     * @return 结果，成功true,失败false
     */
    bool updateParent(FolderContent *data, qint64 parentid);
    /**
     * @brief getChildren 获取子节点信息
     * @param folder 当前节点
     * @param containsDeleted 是否包含被删除在回收站的
     * @return 子节点信息
     */
    QList<FolderContent *> getChildren(FolderContent *folder, bool containsDeleted = false);
    /**
     * @brief getNotes 获取笔记本下笔记
     * @param folder 笔记本数据
     * @param containsDeleted 是否包含被删除在回收站的
     * @return 笔记
     */
    QList<NoteContent *> getNotes(FolderContent *folder, bool containsDeleted = false);
    /**
     * @brief getRecycleNotes 获取回收站笔记
     * @return 笔记数据
     */
    QList<NoteContent *> getRecycleNotes();
    /**
     * @brief loadNote 加载笔记数据，不包含笔记内容
     */
    void loadNote();
    /**
     * @brief loadNoteByThread 通过线程加载笔记数据，不包含笔记内容
     */
    void loadNoteByThread();
    /**
     * @brief loadFolder 加载文件夹数据
     */
    void loadFolder();
    /**
     * @brief loadFolderByThread 通过线程加载文件夹数据
     */
    void loadFolderByThread();
    /**
     * @brief loadAllNoteContent 加载所有笔记内容
     */
    void loadAllNoteContent();
    /**
     * @brief loadAllNoteContentByThread  通过线程加载所有笔记内容
     */
    void loadAllNoteContentByThread();

    /**
     * @brief getAllFolders 获取所有笔记列表
     * @return
     */
    const QList<NoteContent *> getAllNotes();
signals:
    /**
     * @brief loadDataFinsh 数据加载完成信号
     */
    void sigLoadDataFinsh();
    /**
     * @brief loadAllNoteContentFinsh 所有笔记内容加载完成信号
     */
    void sigLoadAllNoteContentFinsh();
protected:
    explicit DataManager(QObject *parent = nullptr);
private:
    LoadStates m_loadstate;
    FolderContent *m_recycleFolder {nullptr};
    QHash<qint64, NoteContent *> m_notes;
    QHash<qint64, FolderContent *> m_folders;
};
#endif // DATAMANAGER_H
