/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "dbmanager.h"
#include "utils.h"

#include <QStandardPaths>
#include <QDir>
#include <QDebug>
#include <QSqlError>

static const QStringList create_tables = {
    "CREATE TABLE IF NOT EXISTS note_folder( \
    id INTEGER PRIMARY KEY AUTOINCREMENT,\
    type  INT NOT NULL,\
    parent  INT DEFAULT NULL,\
    delete_flag BOOLEAN DEFAULT 0,\
    name TEXT NOT NULL,\
    children TEXT DEFAULT NULL,\
    icon TEXT DEFAULT NULL,\
    password_text TEXT DEFAULT NULL,\
    create_time timestamp NOT NULL,\
    modify_time timestamp DEFAULT 0,\
    delete_time timestamp DEFAULT 0)",

    "CREATE TABLE IF NOT EXISTS note(\
    id INTEGER PRIMARY KEY AUTOINCREMENT,\
    parent  INT NOT NULL,\
    type INT NOT NULL DEFAULT 0,\
    top BOOLEAN DEFAULT 0,\
    delete_flag BOOLEAN DEFAULT 0,\
    name TEXT NOT NULL,\
    content TEXT NOT NULL,\
    tags TEXT DEFAULT NULL,\
    password_text TEXT DEFAULT NULL,\
    font_size INT DEFAULT 0,\
    font_name TEXT DEFAULT NULL,\
    synopsis_content TEXT DEFAULT NULL,\
    synopsis_icon TEXT DEFAULT NULL,\
    create_time timestamp NOT NULL,\
    modify_time timestamp DEFAULT 0,\
    delete_time timestamp DEFAULT 0,\
    top_time timestamp DEFAULT 0)"
};

DbManager::DbManager(QObject *parent) : QObject(parent)
{
    initDB();
}

DbManager::~DbManager()
{
    if (m_dbbase.isOpen()) {
        m_dbbase.close();
    }
}

DbManager *DbManager::getInstance()
{
    static DbManager instance;
    return &instance;
}

bool DbManager::initDB()
{
    QString vnoteDatabasePath = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);

    QFileInfo dbDir(vnoteDatabasePath);
    dbDir.setFile(vnoteDatabasePath + QDir::separator());

    if (!dbDir.exists()) {
        Utils::getInstance()->mkMutiDir(vnoteDatabasePath);
        qInfo() << "Create vnote db directory:" << vnoteDatabasePath;
    }

    QString vnoteDatebaseName = "deepin-voice-note2.0.db";

    QString vnoteDbFullPath = dbDir.filePath() + vnoteDatebaseName;

    if (QSqlDatabase::contains(vnoteDatebaseName)) {
        m_dbbase = QSqlDatabase::database(vnoteDatebaseName);
    } else {
        m_dbbase = QSqlDatabase::addDatabase("QSQLITE", vnoteDatebaseName);
        m_dbbase.setDatabaseName(vnoteDbFullPath);
    }

    if (!m_dbbase.open()) {
        qCritical() << "Open database failed:" << m_dbbase.lastError().text();

        return m_isDbInitOK;
    }

    qInfo() << "Database opened:" << vnoteDbFullPath;

    m_isDbInitOK = true;

    for (auto &it : create_tables) {
        operationDb(it);
    }

    return m_isDbInitOK;
}

bool DbManager::operationDb(const QString &sql, std::function<void (QSqlQuery &)> resultCallback)
{
    if (!m_isDbInitOK) {
        qCritical() << "db not init ok.";
        return false;
    }
    QSqlQuery sqlQuery = QSqlQuery(m_dbbase);
    m_dbLock.lock();
    if (!sqlQuery.exec(sql)) {
        qCritical() << sql << "operation failed error: " << sqlQuery.lastError().text();
        m_dbLock.unlock();
        return false;
    }
    m_dbLock.unlock();
    if (nullptr != resultCallback) {
        resultCallback(sqlQuery);
    }
    return true;
}

void DbManager::checkSqlStr(QString &sql)
{
    sql.replace("'", "''");
}
