/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "audioplaymanager.h"
#include "vlcpalyer.h"
#include "utils.h"
#include "statesmanager.h"
#include "praseaudiowaveform.h"

#include <QPainter>
#include <QFileInfo>

#include <DGuiApplicationHelper>

DGUI_USE_NAMESPACE

AudioPlayManager::AudioPlayManager(QQuickItem *parent)
    : QQuickPaintedItem(parent)
{
    setAcceptedMouseButtons(Qt::AllButtons);
}

QString AudioPlayManager::getAudioTitle()
{
    return m_audioInfo.title;
}

void AudioPlayManager::setAudioTitle(const QString &title)
{
    if (m_audioInfo.title != title) {
        m_audioInfo.title = title;
        emit sigAudioTitleChange();
    }
}

QString AudioPlayManager::getPlayTime()
{
    return m_playTime;
}

void AudioPlayManager::setPlayTime(const QString &time)
{
    m_playTime = time;
    emit sigPlayTimeChange();
}

int AudioPlayManager::slotStartPlay(const QString &data, bool isReset)
{
    if (isReset) {
        if (!Utils::getInstance()->parseVoiceInfo(data, m_audioInfo)) {
            return 1;
        }
        QFileInfo info(m_audioInfo.voicePath);
        if (!info.exists()) {
            return 2;
        }
        if (nullptr == m_player) {
            m_player = new VlcPalyer(this);
            connect(m_player, &VlcPalyer::durationChanged, this, &AudioPlayManager::slotDurationChanged, Qt::QueuedConnection);
            connect(m_player, &VlcPalyer::positionChanged, this, &AudioPlayManager::slotCurrentPosChanged, Qt::QueuedConnection);
            connect(m_player, &VlcPalyer::playEnd, this, &AudioPlayManager::slotPlayFinsh, Qt::QueuedConnection);
        }
        m_currentPos = 0;
        //采集语音文件波形
        if (m_audioScaleSamples.find(m_audioInfo.voicePath) == m_audioScaleSamples.end()) {
            PraseAudioWaveform *prase = new PraseAudioWaveform(this);
            prase->setMaxSamples(1000);
            prase->setMaxWaveformHeight(clipRect().height() - 12);
            prase->setAudioInfo(m_audioInfo);
            connect(prase, &PraseAudioWaveform::sigPraseFinsh, this, &AudioPlayManager::slotPraseWaveformFinsh);
            QMetaObject::invokeMethod(prase, "startPrase", Qt::QueuedConnection);
        }
        m_player->setFilePath(m_audioInfo.voicePath);
        m_strDuration = QDateTime::fromTime_t(static_cast<uint>(m_audioInfo.voiceSize / 1000)).toUTC().toString("mm:ss");
        updatePlayTime();
        emit sigAudioTitleChange();
        m_player->play();
        StatesManager::instance()->setState(StatesManager::StatePlaying, true);
        return 0;
    }

    VlcPalyer::VlcState status = m_player->getState();
    if (status == VlcPalyer::Playing) {
        m_player->pause();
    } else {
        m_player->play();
    }
    return 0;
}

void AudioPlayManager::slotDurationChanged(qint64 duration)
{
    m_audioInfo.voiceSize = duration;
}

void AudioPlayManager::slotCurrentPosChanged(qint64 pos)
{
    if (m_sliderReleased == true) {
        m_currentPos = pos;
        update();
        updatePlayTime();
    }
}

void AudioPlayManager::slotPlayFinsh()
{
    m_player->stop();
    m_sliderReleased = true;
    m_currentPos = m_audioInfo.voiceSize;
    updatePlayTime();
    update();
    StatesManager::instance()->setState(StatesManager::StatePlaying, false);
    emit sigPlayFinsh();
}

bool AudioPlayManager::updatePlayTime()
{
    QString time = QDateTime::fromTime_t(static_cast<uint>(m_currentPos / 1000)).toUTC().toString("mm:ss") + "/" + m_strDuration;
    if (m_playTime == time) {
        return false;
    }
    setPlayTime(time);
    return true;
}

void AudioPlayManager::paint(QPainter *painter)
{
    painter->save();
    QRectF rc = clipRect();
    painter->fillRect(QRectF(0, 1, rc.width(), rc.height() - 2),
                      DGuiApplicationHelper::instance()->applicationPalette().color(DPalette::ObviousBackground));
    qreal currentXPos = 0.0;
    if (m_audioInfo.voiceSize) {
        currentXPos = m_currentPos * rc.width() / m_audioInfo.voiceSize;
    }

    //绘制波形
    auto it = m_audioScaleSamples.find(m_audioInfo.voicePath);
    if (it != m_audioScaleSamples.end()) {
        int count = it.value().size();
        qreal tmpPos = 0.0;
        int step = int(count * 3 / rc.width());
        QColor startColor(0x00, 0X79, 0XFF);
        QColor endColor(0X03, 0XD0, 0XD6);
        for (int i = 0; i < count; i += step) {
            if (tmpPos > rc.right()) {
                break;
            }
            QColor color;
            if (tmpPos > currentXPos) {
                color =  QColor(0xB0, 0xB0, 0xB0);
            } else {
                int R = startColor.red() + i * (endColor.red() - startColor.red()) / count;
                int G = startColor.green() + i * (endColor.green() - startColor.green()) / count;
                int B = startColor.blue() + i * (endColor.blue() - startColor.blue()) / count;
                color =  QColor(R, G, B);
            }
            painter->fillRect(QRectF(tmpPos, (rc.height() - it.value()[i]) / 2, 2, it.value()[i]), color);
            tmpPos += 3.0;
        }
    }
    //绘制播放进度条
    if (m_audioInfo.voiceSize) {
        qreal maxRight = rc.width() - 3;
        painter->fillRect(QRectF(qMin(maxRight, currentXPos), 0, 3, rc.height()), Qt::red);
    }
    painter->restore();
}

void AudioPlayManager::mousePressEvent(QMouseEvent *event)
{
    Q_UNUSED(event)
    m_sliderReleased = false;
}

void AudioPlayManager::mouseMoveEvent(QMouseEvent *event)
{
    if (!m_player || (m_player->getState() != VlcPalyer::Paused && m_player->getState() != VlcPalyer::Playing)) {
        return;
    }
    m_currentPos = qint64(m_audioInfo.voiceSize * event->x() / clipRect().width());
    update();
    updatePlayTime();
}

void AudioPlayManager::mouseReleaseEvent(QMouseEvent *event)
{
    m_sliderReleased = true;
    if (nullptr == m_player) {
        return;
    }
    VlcPalyer::VlcState state = m_player->getState();
    if (state == VlcPalyer::Playing || state == VlcPalyer::Paused) {
        m_currentPos = qint64(m_audioInfo.voiceSize * event->x() / clipRect().width());
        if (m_currentPos >= m_audioInfo.voiceSize) {
            slotPlayFinsh();
        } else {
            m_player->setPosition(m_currentPos);
        }
    }
}

void AudioPlayManager::slotPraseWaveformFinsh(const QString &url, const QList<qreal> &data)
{
    m_audioScaleSamples.insert(url, data);
    update();
}
