/*
* Copyright (C) 2022 Uniontech Software Technology Co.,Ltd.
*
* Author:     liuyanga <liuyanga@uniontech.com>
*
* Maintainer: liuyanga <liuyanga@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "audiorecordmanager.h"
#include "gstreamrecorder.h"
#include "audiowatcher.h"
#include "utils.h"
#include "notedata.h"
#include "statesmanager.h"

#include <unistd.h>

#include <QPainter>

#include <DGuiApplicationHelper>

DGUI_USE_NAMESPACE

static const qreal stepWidth = 3.0;

AudioRecordManager::AudioRecordManager(QQuickItem *parent)
    : QQuickPaintedItem(parent)
{
    initRecordPath();
    m_audioRecoder = new GstreamRecorder(this);
    connect(m_audioRecoder, SIGNAL(audioBufferProbed(const QAudioBuffer &)),
            this, SLOT(slotAudioBufferProbed(const QAudioBuffer &)));
    m_audioWatcher = AudioWatcher::instance();
    connect(m_audioWatcher, &AudioWatcher::sigDeviceChange, this, &AudioRecordManager::slotDeviceChange);
}

bool AudioRecordManager::slotStartRecord()
{
    if (StatesManager::instance()->getState(StatesManager::StateRecording)) {
        return m_audioRecoder->startRecord();
    }
    QString device = m_audioWatcher->getDeviceName(AudioWatcher::Micphone);
    if (device.isEmpty()) {
        return false;
    }
    m_audioRecoder->setDevice(device);
    m_recordName = QDateTime::currentDateTime()
                   .toString("yyyyMMddhhmmss")
                   + ".mp3";
    m_recordMsec = 0;
    m_audioRecoder->setOutputFile(m_recordDir + m_recordName);
    setRecordTime("00:00");
    bool ret = m_audioRecoder->startRecord();
    StatesManager::instance()->setState(StatesManager::StateRecording, ret);
    return  ret;
}

void AudioRecordManager::slotPauseRecord()
{
    m_audioRecoder->pauseRecord();
}

void AudioRecordManager::slotStopRecord()
{
    if (!StatesManager::instance()->getState(StatesManager::StateRecording)) {
        return;
    }
    m_audioRecoder->stopRecord();
    QFile f(m_recordDir + m_recordName);
    if (f.open(QIODevice::ReadWrite | QIODevice::Text)) {
        f.flush(); //将用户缓存中的内容写入内核缓冲区
        fsync(f.handle()); //将内核缓冲写入磁盘
        f.close();
    }
    VoiceInfo info;
    info.voicePath = m_recordName;
    info.voiceSize = m_recordMsec;
    info.createTime = QDateTime::currentDateTime();
    info.title = info.createTime.toString("yyyyMMdd hh.mm.ss");
    emit sigRecordFinsh(Utils::getInstance()->makeVoiceInfo(info));
    setRecordTime("00:00");
    StatesManager::instance()->setState(StatesManager::StateRecording, false);
    m_maxSamples = 0.01;
    m_audioScaleSamples.clear();
    update();
}

void AudioRecordManager::paint(QPainter *painter)
{
    painter->save();
    QRectF rc = clipRect();
    painter->fillRect(QRectF(0, 1, rc.width(), rc.height() - 2), DGuiApplicationHelper::instance()->applicationPalette().color(DPalette::ObviousBackground));
    qreal halfWidth = rc.width() / 2;
    if (!m_audioScaleSamples.size()) {
        painter->fillRect(QRectF(halfWidth, 0, stepWidth, rc.height()), Qt::red);
        painter->restore();
        return;
    }
    qreal xPos = halfWidth;
    int count = m_audioScaleSamples.size() - 1;
    while (xPos > 0 && count >= 0) {
        painter->fillRect(QRectF(xPos, (rc.height() - m_audioScaleSamples[count]) / 2, 2, m_audioScaleSamples[count]), Qt::red);
        count -= 1;
        xPos -= 3;
    }

    painter->fillRect(QRectF(halfWidth, 0, stepWidth, rc.height()), Qt::red);
    xPos = halfWidth + stepWidth;
    while (xPos < rc.width()) {
        painter->fillRect(QRectF(xPos, (rc.height() - 1) / 2, 1, 1), Qt::red);
        xPos += 3;
    }
    painter->restore();
}

void AudioRecordManager::setRecordTime(const QString &recordTime)
{
    if (m_recordTime != recordTime) {
        m_recordTime = recordTime;
        emit sigRecordTimeChange();
    }
}

QString AudioRecordManager::getRecordTime()
{
    return m_recordTime;
}

void AudioRecordManager::initRecordPath()
{
    m_recordDir = Utils::getInstance()->getAudioSavePath();
}

void AudioRecordManager::slotAudioBufferProbed(const QAudioBuffer &buffer)
{
    qint64 msec = buffer.startTime();
    if (msec != m_recordMsec) {
        setRecordTime(formatRecordTime(msec));
        m_recordMsec = msec;
        if (m_recordMsec > (60 * 60 * 1000)) {
            slotStopRecord();
            return;
        }
    }
    upDateWaveForm(buffer);

}

void AudioRecordManager::upDateWaveForm(const QAudioBuffer &buffer)
{
    qreal ret = qAbs(Utils::getInstance()->getAudioWaveform(buffer));
    if (ret > m_maxSamples) {
        m_maxSamples = ret;
    }
    ret = (ret / m_maxSamples) * (clipRect().height() - 12);
    m_audioScaleSamples.push_back(ret > 1.0 ? ret : 1);
    if (m_audioScaleSamples.size() > 1000) {
        m_audioScaleSamples.removeFirst();
    }
    update();
}


QString AudioRecordManager::formatRecordTime(qint64 millisecond)
{
    qint64 curSecond = millisecond / 1000;
    if (curSecond < 3600) {
        return QDateTime::fromTime_t(static_cast<uint>(curSecond)).toUTC().toString("mm:ss");
    }
    return QString("60:00");
}

void AudioRecordManager::slotDeviceChange(int mode)
{
    if (mode == AudioWatcher::Micphone) {
        slotStopRecord();
    }
}
